
precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform lowp vec4 hairColor;
uniform float singleStepV;
uniform float singleStepH;
uniform float ratio;
uniform float brightnessRatio;

const float MAX_GRADIENT_STEP=6.0;
const lowp vec3 weight = vec3(0.299,0.587,0.114);
const highp vec4 blend=vec4(1.0,0.3333333,0.0,1.0);

bool isHairColor(vec2 coord){
    vec4 color=texture2D(inputImageTexture2, coord);
    return color.r >= 0.1;
}

bool isEdge(vec2 coord){
    bool curCoord=isHairColor(coord);
    bool leftTop    =isHairColor(coord+MAX_GRADIENT_STEP*vec2(-singleStepH,-singleStepV));
    if(leftTop!=curCoord){
        return true;
    }
    bool leftBottom =isHairColor(coord+MAX_GRADIENT_STEP*vec2(-singleStepH, singleStepV));
    if(leftBottom!=curCoord){
        return true;
    }
    bool rightTop   =isHairColor(coord+MAX_GRADIENT_STEP*vec2(singleStepH, -singleStepV));
    if(rightTop!=curCoord){
        return true;
    }
    bool rightBottom=isHairColor(coord+MAX_GRADIENT_STEP*vec2(singleStepH,  singleStepV));
    if(rightBottom!=curCoord){
        return true;
    }
    return false;
}
vec3 rgb2HSV(vec3 rgb){
    float R=rgb.x;
    float G=rgb.y;
    float B=rgb.z;
    float H;
    float S;
    float V;
    float max=max(max(R,G),B);
    float min=min(min(R,G),B);
    
    if(max==min&&max==0.0){
        return vec3(360.0,0.0,0.0);
    }
    if(max==min&&max==1.0){
        return vec3(360.0,0.0,1.0);
    }
    if(R ==max){
        H = (G-B)/(max-min);
    }
    if (G ==max){
        H = 2.0 + (B-R)/(max-min);
    }
    if (B == max){
        H = 4.0 + (R-G)/(max-min);
    }
    H = H * 60.0;
    if( H < 0.0){
        H = H + 360.0;
    }
    S=(max-min)/max;
    V=max;
    return vec3(H,S,V);
}

vec3 hsv2RGB(vec3 hsv){
    float h=hsv.x;
    float s=hsv.y;
    float v=hsv.z;
    float r;
    float g;
    float b;
    h =h / 60.0;            // sector 0 to 5
    float i = floor( h );
    float f = h - i;          // factorial part of h
    float p = v * ( 1.0 - s );
    float q = v * ( 1.0 - s * f );
    float t = v * ( 1.0 - s * ( 1.0 - f ) );
    if(int(i)==0){
        r = v;
        g = t;
        b = p;
    }
    else if(int(i)== 1){
        r = q;
        g = v;
        b = p;
    }
    else if(int(i)== 2){
        r = p;
        g = v;
        b = t;
    }
    else if(int(i)==3){
        r = p;
        g = q;
        b = v;
    }
    else if(int(i)==4){
        r = t;
        g = p;
        b = v;
    }
    else{        // case 5:
        r = v;
        g = p;
        b = q;
    }
    return vec3(r,g,b);
}

float computeGradientRatio(vec2 coord){
    vec2 start=coord+MAX_GRADIENT_STEP*vec2(-singleStepH,-singleStepV);
    vec2 curCoord;
    vec4 curColor;
    float totalHairColorCount=0.0;;
    for(float i=0.0;i<MAX_GRADIENT_STEP;i++){
        for(float j=0.0;j<MAX_GRADIENT_STEP;j++){
            curCoord=start+2.0*vec2(i*singleStepH,j*singleStepV);
            curColor=texture2D(inputImageTexture, curCoord);
            if(isHairColor(curCoord)){
                totalHairColorCount++;
            }
        }
    }
    float ratio=totalHairColorCount/pow(MAX_GRADIENT_STEP,2.0);
    if(ratio>1.0){
        ratio=1.0;
    }
    else if(ratio<0.0){
        ratio=0.0;
    }
    return ratio;
}


vec4 blendSoft(vec4 base, vec4 overlay){
    float nr=base.r;
    float ng=base.g;
    float nb=base.b;
    
    if(base.r <= 0.5){
        nr = (2.0 * overlay.r - 1.0) * (sqrt(base.r) - base.r) + base.r;
    }else{
        nr = (2.0 * overlay.r - 1.0)*(base.r - base.r * base.r) + base.r;
    }
    
    if(base.g <= 0.5){
        ng = (2.0 * overlay.g - 1.0) * (sqrt(base.g) - base.g) + base.g;
    }else{
        ng = (2.0 * overlay.g - 1.0)*(base.g - base.g * base.g) + base.g;
    }
    
    if(base.b <= 0.5){
        nb = (2.0 * overlay.b - 1.0) * (sqrt(base.b) - base.b) + base.b;
    }else{
        nb = (2.0 * overlay.b - 1.0)*(base.b - base.b * base.b) + base.b;
    }
    vec4 newColor = vec4(nr, ng, nb, base.a);
    return newColor;
}

vec4 obtainNewHairColor(vec4 src,vec4 dst){
    vec4 blendResult=blendSoft(src,blend);
    vec3 blendResultHSV=rgb2HSV(blendResult.rgb);

    float gray=dot(src.rgb,weight);
    float realS=blendResultHSV.y;
    float realV=blendResultHSV.z;
//    if(brightnessRatio>0.5){
//        realS=pow(realS,3.0*gray*(brightnessRatio-0.5)+1.0);
//        if(realS<0.0){
//            realS=0.0;
//        }
//        else if(realS>1.0){
//            realS=1.0;
//        }
//    }
//    realV=pow(realV,3.0*gray*(0.5-brightnessRatio)+1.0);
//    if(realV<0.0){
//        realV=0.0;
//    }
//    else if(realV>1.0){
//        realV=1.0;
//    }

    vec3 dstHSV=rgb2HSV(dst.rgb);
    vec3 realDstHSV=vec3(dstHSV.x,realS*dstHSV.y,(gray+realV)*0.5*pow(dstHSV.z+brightnessRatio,1.0-(gray+realV)*0.5));
    vec3 realDst=hsv2RGB(realDstHSV);
    vec4 result=vec4(realDst,src.a);
    return result;
}
void main()
{
    vec4 originColor=texture2D(inputImageTexture, textureCoordinate);
    gl_FragColor = originColor;
    vec4 dst=hairColor;
    if(isEdge(textureCoordinate)){
        float gradientRatio=computeGradientRatio(textureCoordinate);
        vec4 result=obtainNewHairColor(originColor,dst);
        gl_FragColor=vec4(mix(originColor.rgb,result.rgb,ratio*gradientRatio),originColor.a);
    }
    else if(isHairColor(textureCoordinate)){
        vec4 result=obtainNewHairColor(originColor,dst);
        gl_FragColor=vec4(mix(originColor.rgb,result.rgb,ratio),originColor.a);
    }
}
